@@ -28,7 +28,25 @@ group = parser.add_mutually_exclusive_group()
 group.add_argument(
     "-e",
     "--edit",
-    help="Use fzf to interactively choose file(s) to edit",
+    help="Use fzf to interactively choose file(s) to open in your $EDITOR",
+    action="store_true",
+)
+group.add_argument(
+    "-a",
+    "--add",
+    help="Use fzf to interactively stage changes to your dofiles",
+    action="store_true",
+)
+group.add_argument(
+    "-r",
+    "--run",
+    help="Interactively choose an excutable file/script to run.",
+    action="store_true",
+    dest="run_executable",
+)
+group.add_argument(
+    "--restore",
+    help="Use fzf to interactively choose file(s) to remove from the staging area.",
     action="store_true",
 )
 group.add_argument(
@@ -37,25 +55,13 @@ group.add_argument(
     help="Show status of dotfiles repo",
     action="store_true",
 )
-group.add_argument(
-    "-a",
-    "--add",
-    help="Interactively add changes your dotfiles",
-    action="store_true",
-)
 group.add_argument(
     "-ls",
     "--list",
-    help="Show list of all files in the repo",
+    help="list all files in the repo",
     action="store_true",
 )
-group.add_argument(
-    "-r",
-    "--restore",
-    help="Use fzf to interactively choose file(s) to remove from the staging area.",
-    action="store_true",
-)
-args = parser.parse_args()
+args, extras = parser.parse_known_args()
 dotfiles = Dotfiles()
 
 if args.edit:
@@ -67,8 +73,12 @@ elif args.status:
 elif args.list:
     for dotfile in dotfiles.list_all:
         print(dotfile)
+elif args.run_executable:
+    dotfiles.run_executable()
 elif args.restore:
     dotfiles.restore()
+elif len(extras) > 1 and extras[0] == "git":
+    dotfiles.git_passthrough(extras)
 else:
     parser.parse_args(["-h"])
 
@@ -82,4 +92,4 @@ else:
 # d. (create) Initiate bare repo and ask which file to append the aliases to?
 #              [.bashrc, .bash_aliases, .bash_profile, .profile, .zshrc]
 
-# vim: foldlevel=4:
+# vim: foldlevel=0:
