@@ -5,7 +5,7 @@
 # Standard Library
 # https://docs.python.org/3/library/functools.html?highlight=functools#functools.cached_property
 from functools import cached_property
-from os import chdir, getenv
+from os import chdir, getenv, access, X_OK
 import re
 from pathlib import Path
 from subprocess import run
@@ -19,6 +19,7 @@ from pydymenu import fzf
 # Project Modules
 from mydot.console import console
 from mydot.exceptions import MissingRepositoryLocation, WorktreeMissing
+from mydot.system_funcs import script_plus_args
 
 
 class Dotfiles:
@@ -26,11 +27,16 @@ class Dotfiles:
 
     def __init__(
         self,
-        git_dir: Union[Path, str, None] = None,
+        local_bare_repo: Union[Path, str, None] = None,
         work_tree: Union[Path, str, None] = None,
     ):
-        """Define a 'git_dir' and 'work_tree' to begin."""
-        self.bare_repo: Path = self._resolve_repo_location(git_dir)
+        """Create a new Dotfiles object to manage your repository.
+
+        When no locations are passed the default locations are:
+        - "$DOTFILES" for the bare repository
+        - "$HOME" for the work tree
+        """
+        self.bare_repo: Path = self._resolve_repo_location(local_bare_repo)
         self.work_tree: Path = self._resolve_work_tree_location(work_tree)
         self._git_base = [
             "git",
@@ -95,8 +101,24 @@ class Dotfiles:
                 run([self.editor, edits[0]])
             else:
                 run([self.editor, "-o"] + edits)
+            self.freshen()
             return edits
 
+    def run_executable(self) -> str:
+        """Interactively choose an executable to run. Optionally add arguements."""
+        exe = fzf(
+            self.executables,
+            prompt="Pick a file to run: ",
+            multi=False,
+            preview=f"{self.preview_app}" + " {}",
+        )
+        if exe is None:
+            sys_exit("No selection made. Cancelling action.")
+        else:
+            command = script_plus_args(Path(self.work_tree) / exe[0])
+            run(command)
+            return str(exe[0])
+
     def add_changes(self) -> List[str]:
         """Interactively choose modified files to add to the staging area."""
         if self.modified_unstaged:
@@ -110,13 +132,16 @@ class Dotfiles:
                 sys_exit("No selection made. No changes will be staged.")
             else:
                 run(self._git_base + ["add", "-v", "--"] + adding)
+                self.freshen()
                 return adding
         else:
             sys_exit("No unstaged changes to 'add'.")
 
     def restore(self) -> List[str]:
         """Interactively choose files to remove from the staging area."""
-        restorables = sorted(list(set(self.modified_staged + self.deleted_staged)))
+        restorables = sorted(
+            list(set(self.modified_staged + self.deleted_staged + self.adds_staged))
+        )
         if restorables:
             restores = fzf(
                 restorables,
@@ -128,6 +153,7 @@ class Dotfiles:
                 sys_exit("No selection made. No files will be unstaged.")
             else:
                 run(self._git_base + ["restore", "--staged", "--"] + restores)
+                self.freshen()
                 return restores
         else:
             self.show_status()
@@ -155,25 +181,29 @@ class Dotfiles:
         reformatted = re.sub(renamed_regex, reformat_renames, output).split("\x00")
         return [statusline for statusline in reformatted if len(statusline) > 0]
 
-    @cached_property
+    @property
     def tracked(self) -> List[str]:
-        cmd = self._git_base + ["ls-tree", "--full-tree", "--full-name", "-r", "HEAD"]
-        lines = run(cmd, text=True, capture_output=True).stdout.strip().split("\n")
-        tracked = [item.split("\t")[-1] for item in lines]
-        # TODO: prune deleted staged, renamed old_names
-        return tracked
+        output_lines = run(
+            self._git_base
+            + ["ls-tree", "--full-tree", "--full-name", "-r", "HEAD", "-z"],
+            text=True,
+            capture_output=True,
+        ).stdout.split("\x00")
+        return [row.split("\t")[-1] for row in output_lines if len(row) > 0]
 
-    @cached_property
+    @property
     def adds_staged(self) -> List[str]:
-        if len(self.short_status) == 0:
-            return []
-        else:
-            # TODO this join seems wrong
-            return [
-                " ".join(fp.split()[1:]) for fp in self.short_status if fp[0] == "A"
-            ]
+        """Returns list of newly added files to the staging area."""
+        return [f[3:] for f in self.short_status if f[0] == "A"]
 
-    @cached_property
+    @property
+    def oldnames(self) -> List[str]:
+        """Returns previous name of files renamed in staging area."""
+        return [
+            stat[3:].split(" -> ")[0] for stat in self.short_status if stat[0] == "R"
+        ]
+
+    @property
     def renames(self) -> List[str]:
         if len(self.short_status) == 0:
             return []
@@ -187,12 +217,45 @@ class Dotfiles:
 
     @cached_property
     def list_all(self) -> List[str]:
-        # TODO pseudocode:
-        # (tracked - (deletes + oldnames)) + renames + adds
-        adds = self.adds_staged
-        tracked = self.tracked
-        renames = self.renames
-        return sorted(adds + tracked + renames)
+        """List of all files in repo [+new adds, -things going ]. (relative paths)"""
+        include = self.tracked + self.adds_staged + self.renames
+        removed = self.deleted_staged + self.oldnames
+        all = [f for f in include if f not in removed]
+        return sorted(list(set(all)))
+
+    @property
+    def list_all_as_path(self) -> List[Path]:
+        """List of Path() objects for each file in the repository."""
+        return [(Path(self.work_tree) / p) for p in self.list_all]
+
+    @cached_property
+    def executables(self) -> List[str]:
+        """List of executable dotfiles (relative paths)"""
+        return [
+            str(Path(e).relative_to(self.work_tree))
+            for e in self.list_all_as_path
+            if access(e, X_OK)
+        ]
+
+    def freshen(self) -> None:
+        """Deltes @cached_property values which forces recompute on next use.
+
+        Very useful for programs building upon Dotfiles. Allows you to take actions,
+        utilize @cached_property values during runtime but request caches be dropped
+        so further runs will request new data from `git`
+        """
+        try:
+            del self.short_status
+        except AttributeError:
+            pass  # ignore failure to delete uncached functions
+        try:
+            del self.list_all
+        except AttributeError:
+            pass
+        try:
+            del self.executables
+        except AttributeError:
+            pass
 
     @property
     def deleted_staged(self) -> List[str]:
@@ -238,5 +301,9 @@ class Dotfiles:
             else:
                 sys_exit("Cannot find a suitable editor, and boy did we look!")
 
+    def git_passthrough(self, args: List[str]):
+        """Send commands to git with --git-dir and --work-tree set."""
+        run(self._git_base + args[1:])
+
 
 # vim: foldlevel=1 :
